/* globals Property, raisedEvent, localEndpoint, cloudService, registerTrigger, unregisterTrigger, Logger, putTriggerParameter, putConditionParameter, dSID2dSUID */
// 'use strict';

var LOGFILE_NAME = 'event-push-notification.log';
var lInfo = new Logger(LOGFILE_NAME);
var lWarn = new Logger(LOGFILE_NAME);
var ALL_RECEIVERS = '*';
var DSID_LENGTH = 24;
var actions = {
    FETCH_DEVICE_LIST: 'fetchDeviceList',
    EXECUTE: 'execute',
    TEST: 'test',
    SEND: 'send',
    SAVE: 'save',
    ENABLE: 'enable',
    DISABLE: 'disable',
    DELETE: 'delete'
};

/* eslint-disable */
var Utf8 = {
    // public method for url encoding
    encode: function (string) {
        string = string.replace(/\r\n/g, '\n');
        var utftext = '';
        for (var n = 0; n < string.length; n++) {
            var c = string.charCodeAt(n);
            if (c < 128) {
                utftext += String.fromCharCode(c);
            } else if (c > 127 && c < 2048) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            } else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }

        }
        return utftext;
    },

    // public method for url decoding
    decode: function (utftext) {
        var string = '';
        var i = 0;
        var c;
        var c2;
        var c3;
        c = c2 = c3 = 0;
        while (i < utftext.length) {
            c = utftext.charCodeAt(i);
            if (c < 128) {
                string += String.fromCharCode(c);
                i++;
            } else if (c > 191 && c < 224) {
                c2 = utftext.charCodeAt(i + 1);
                string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                i += 2;
            } else {
                c2 = utftext.charCodeAt(i + 1);
                c3 = utftext.charCodeAt(i + 2);
                string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                i += 3;
            }
        }
        return string;
    }
};
/* eslint-enable */

function storeProperty (path, value) {
    Property.setProperty(path, value);
    Property.setFlag(path, 'ARCHIVE', true);
}

function checkAndUpdateData () {
    lInfo.logln('checkAndUpdateData');
    var updates = [];
    var entriesMainNode = Property.getNode('values');
    if (!entriesMainNode) {
        return;
    }
    var entries = entriesMainNode.getChildren();
    for (var i = 0; i < entries.length; i++) {
        var entry = entries[i];
        var triggersNode = entry.getChild('triggers');
        if (triggersNode) {
            var triggers = triggersNode.getChildren() || [];
            triggers.forEach(function (trigger, i) {
                switch (trigger.getChild('type').getValue()) {
                    case 'device-msg': // intended fallthrough
                    case 'device-binary-input': // intended fallthrough
                    case 'device-scene': // intended fallthrough
                    case 'device-sensor':
                        if (trigger.getChild('dsuid') == null) {
                            lInfo.logln('--- converting dsid to dsuid ' + i);
                            storeProperty(trigger.getPath() + '/dsuid', dSID2dSUID(trigger.getChild('dsid').getValue()));
                            updates.push('dsid to dsuid for trigger ' + i);
                        }
                        break;
                    case 'state-change':
                        var triggerName = trigger.getChild('name').getValue();
                        var nameParts = triggerName.split('.');
                        if (nameParts[0] === 'dev' && nameParts[1].length === DSID_LENGTH) {
                            var dsuid = dSID2dSUID(nameParts[1]);
                            storeProperty(trigger.getPath() + '/name', nameParts[0] + '.' + dsuid + '.' + nameParts[2]);
                            updates.push('dsid to dsuid for trigger ' + i);
                        }
                        break;
                    default:
                        break;
                }
            });
        }
        var conditionsNode = entry.getChild('conditions');
        var statesNode = conditionsNode && conditionsNode.getChild('states');
        if (statesNode) {
            var states = statesNode.getChildren() || [];
            states.forEach(function (state, i) {
                var stateName = state.getName();
                var stateValue = state.getValue();
                var stateNameParts = stateName.split('.');
                if (stateNameParts[0] === 'dev' && stateNameParts[1].length === DSID_LENGTH) {
                    var dsuid = dSID2dSUID(stateNameParts[1]);
                    var newStateName = stateNameParts[0] + '.' + dsuid + '.' + stateNameParts[2];
                    storeProperty(statesNode.getPath() + '/' + newStateName, stateValue);
                    statesNode.removeChild(stateName);
                    updates.push('dsid to dsuid for condition state ' + i);
                }
            });
        }
    }
    if (updates.length) {
        lInfo.logln('Migrated: \n' + updates.join(' | '));
        Property.store();
    }
}

function storeValue (valueJsonString) {
    var value = JSON.parse(unescape(valueJsonString));
    var id = value.id;

    var entriesMainNode = Property.getNode('values');
    if (id == null) { // find first available id
        id = 1;
        var entries = entriesMainNode && entriesMainNode.getChildren() || [];
        entries.forEach(function (entry) {
            var entryId = parseInt(entry.getName(), 10);
            if (id <= entryId) {
                id = entryId + 1;
            }
        });
    } else if (entriesMainNode != null) {   // remove existing entry
        try {
            entriesMainNode.removeChild(id.toString());
        } catch (e) {
            lWarn.logln('warning: Cannot remove node with id ' + id);
        }
    }

    storeProperty('values/' + id + '/name', Utf8.encode(unescape(value.name)));
    storeProperty('values/' + id + '/id', id);
    storeProperty('values/' + id + '/message', Utf8.encode(unescape(value.message)));
    value.uuid && value.uuid.forEach(function (uuid, i) {
        storeProperty('values/' + id + '/uuid/' + i, uuid);
    });
    putTriggerParameter('values/' + id, value);
    putConditionParameter('values/' + id, value);
    if (value.metaData != null) {
        storeProperty('values/' + id + '/metaData', value.metaData);
    }
    Property.store();

    updateTriggerRegistration(Property.getNode('values/' + id));
}

function updateTriggerRegistration (entryNode) {
    var parameter = { actions: 'execute' };
    lInfo.logln('EP-Config Set Trigger ' + entryNode.getPath());
    registerTrigger(entryNode.getPath(), 'event-push-notification', parameter);
}

function updateAllTriggers () {
    Property.load();
    var entriesMainNode = Property.getNode('values');
    if (!entriesMainNode) {
        lInfo.logln('EP-Config update all triggers : no entries to update');
        return;
    }
    var entries = entriesMainNode.getChildren();
    entries && entries.forEach(function (entry, i) {
        lInfo.logln('EP-Config Set Trigger' + i);
        updateTriggerRegistration(entry);
    });
}

function addLeadingZeroes (number, count) {
    var paddedString = number.toString();
    while (paddedString.length < count) {
        paddedString = '0' + paddedString;
    }
    return paddedString;
}

function timeStamp () {
    var oDate = new Date();
    return oDate.getDate() + '.' + (oDate.getMonth() + 1) + '.' +
        oDate.getFullYear() + ' ' + oDate.getHours() + ':' +
        addLeadingZeroes(oDate.getMinutes(), 2);
}

function replaceTags (text) {
    return text.replace('%timestamp%', timeStamp());
}

function getNotificationTargetUuids (node) {
    var uuidsNode = node.getChild('uuid');
    return uuidsNode
        ? uuidsNode
            .getChildren()
            .map(function (uuidNode) {
                return uuidNode.getValue();
            })
        : [];
}

function getDevicesByTargetType () {
    var allTargets = { cloudTargets: [], localEndpointTargets: [] };
    return Property.getNode('devices')
        ? Property.getNode('devices').getChildren()
            .reduce(function (allDevices, deviceNode) {
                var targetType = deviceNode.getChild('localEndpoint') != null ? 'localEndpointTargets' : 'cloudTargets';
                allDevices[targetType].push(deviceNode.getChild('uuid').getValue());
                return allDevices;
            }, allTargets)
        : allTargets;
}

function sendPath (node) {
    var messageNode = node.getChild('message');
    var text = messageNode ? messageNode.getValue() : '';
    text = escape(replaceTags(Utf8.decode(text)));

    var uuids = getNotificationTargetUuids(node);
    var devices = getDevicesByTargetType();

    var targetAllApps = uuids.indexOf(ALL_RECEIVERS) !== -1;
    lInfo.logln(targetAllApps ? 'All Targets' : 'Selected Targets');

    var cloudTargets;
    var localEndpointTargets;
    if (targetAllApps) {
        cloudTargets = devices.cloudTargets;
        localEndpointTargets = devices.localEndpointTargets;
    } else {
        cloudTargets = uuids.filter(function (uuid) {
            return devices.cloudTargets.indexOf(uuid) !== -1;
        });
        localEndpointTargets = uuids.filter(function (uuid) {
            return devices.localEndpointTargets.indexOf(uuid) !== -1;
        });
    }

    lInfo.logln('Cloud service targets: ' + JSON.stringify(cloudTargets));
    lInfo.logln('Local endpoint targets: ' + (targetAllApps ? 'ALL' : JSON.stringify(localEndpointTargets)));
    lInfo.logln('Text: ' + text);
    cloudService.sendNotification(cloudTargets, text);
    localEndpoint.sendNotification(targetAllApps ? ALL_RECEIVERS : localEndpointTargets, text);
}

function fetchDeviceList () {
    var evt;
    try {
        lInfo.logln('start fetchDeviceList');
        if (cloudService.getOSPToken() == null) {
            lInfo.logln('Remote Connectivity not ready! just for 5 minutes');
            evt = new Event('event-push-notification', { actions: 'fetchDeviceList', time: '+300' });
            evt.raise();
            return;
        }
        cloudService.fetchNotificationList(function (receivers, errors) {
            if (errors && errors.length) {
                lInfo.logln('errors: ' + JSON.stringify(errors));
                evt = new Event('event-push-notification.saved', { errors: JSON.stringify(errors) });
                evt.raise();
                return;
            }

            lInfo.logln('getting Listindex: ' + receivers.length);
            Property.getNode('/scripts/event-push-notification').removeChild('devices');
            receivers.forEach(function (receiver, index) {
                storeProperty('devices/' + index + '/name', receiver.MobileName);
                storeProperty('devices/' + index + '/uuid', receiver.MobileAppUUID);
                storeProperty('devices/' + index + '/appName', receiver.AppName);
                if (receiver.localEndpoint) {
                    storeProperty('devices/' + index + '/localEndpoint', receiver.localEndpoint);
                }
            });
            Property.store();
            evt = new Event('event-push-notification.saved', {});
            evt.raise();
        });
    } catch (e) {
        lWarn.logln('fetchDeviceList error ' + e);
        evt = new Event('event-push-notification.saved', { errors: JSON.stringify([ e ]) });
        evt.raise();
    }
}

function toggleEntries (ids, action) {
    ids.forEach(function (id) {
        var nodeExists = Property.getNode('values/' + id);
        if (nodeExists) {
            storeProperty('values/' + id + '/conditions/enabled', action === 'enable');
        }
    });
    Property.store();
}

function deleteEntries (ids) {
    var entriesMainNode = Property.getNode('values');
    if (entriesMainNode) {
        var mainPath = entriesMainNode.getPath() + '/';
        ids.forEach(function (id) {
            entriesMainNode.removeChild(id);
            unregisterTrigger(mainPath + id);
        });
        Property.store();
    }
}

function parseIds (idsAsJsonString) {
    var ids = JSON.parse(unescape(idsAsJsonString));
    return typeof ids === 'object'
        ? ids
        : [ ids ];
}

if (raisedEvent.name === 'event-push-notification') {
    Property.load();

    if (raisedEvent.parameter.actions === actions.FETCH_DEVICE_LIST) {
        fetchDeviceList();
    }

    if (raisedEvent.parameter.actions === actions.EXECUTE) {
        lInfo.logln('executing action: ' + raisedEvent.parameter.path);
        var entryToBeExecuted = Property.getNode(raisedEvent.parameter.path);
        if (entryToBeExecuted) {
            try {
                sendPath(entryToBeExecuted);
            } catch (e) {
                lWarn.logln(e);
            }
        } else {
            lWarn.logln('Path not found');
        }
    }

    if (raisedEvent.parameter.actions === actions.TEST) {
        lInfo.logln('testing action id: ' + raisedEvent.parameter.id);
        var entryToBeTested = Property.getNode('values/' + raisedEvent.parameter.id);
        if (entryToBeTested != null) {
            try {
                sendPath(entryToBeTested);
            } catch (e) {
                lWarn.logln(e);
            }
        } else {
            lWarn.logln('Path not found');
        }
    }

    if (raisedEvent.parameter.actions === actions.SEND) {
        var deviceNodeIndex = raisedEvent.parameter.deviceIndex;
        var text = escape(replaceTags(unescape(raisedEvent.parameter.text)));

        var targetAllApps = deviceNodeIndex === ALL_RECEIVERS;
        lInfo.logln('Will send ' + unescape(raisedEvent.parameter.text) + ' to ' + (targetAllApps ? 'all targets' : 'selected targets'));

        var cloudTargets;
        var localEndpointTargets;
        if (targetAllApps) {
            var devices = getDevicesByTargetType();
            cloudTargets = devices.cloudTargets;
            localEndpointTargets = devices.localEndpointTargets;
        } else {
            var deviceNode = Property.getNode('devices/' + deviceNodeIndex);
            var uuid = deviceNode.getChild('uuid').getValue();
            var localEndpointTarget = deviceNode.getChild('localEndpoint') != null;
            cloudTargets = localEndpointTarget ? [] : [ uuid ];
            localEndpointTargets = localEndpointTarget ? [ uuid ] : [];
        }

        lInfo.logln('Cloud service targets: ' + JSON.stringify(cloudTargets));
        lInfo.logln('Local endpoint targets: ' + JSON.stringify(localEndpointTargets));
        lInfo.logln('Text: ' + text);
        cloudService.sendNotification(cloudTargets, text);
        localEndpoint.sendNotification(localEndpointTargets, text);
    }

    if (raisedEvent.parameter.actions === actions.SAVE) {
        lInfo.logln('saving value ' + unescape(raisedEvent.parameter.value));
        var saveErrors = {};
        try {
            storeValue(raisedEvent.parameter.value);
        } catch (e) {
            saveErrors = { errors: JSON.stringify([ e ]) };
            lWarn.logln(e);
        }
        new Event('event-push-notification.saved', saveErrors).raise();
    }

    if (raisedEvent.parameter.actions === actions.ENABLE || raisedEvent.parameter.actions === actions.DISABLE) {
        lInfo.logln(raisedEvent.parameter.actions + ' entry(ies) ' + unescape(raisedEvent.parameter.id));
        var toggleErrors = {};
        try {
            var toggleIds = parseIds(raisedEvent.parameter.id);
            toggleEntries(toggleIds, raisedEvent.parameter.actions);
        } catch (e) {
            toggleErrors = { errors: JSON.stringify([ e ]) };
            lWarn.logln(e);
        }
        new Event('event-push-notification.saved', toggleErrors).raise();
    }

    if (raisedEvent.parameter.actions === actions.DELETE) {
        lInfo.logln('deleting entry(ies) ' + unescape(raisedEvent.parameter.id));
        var deleteErrors = {};
        try {
            var deleteIds = parseIds(raisedEvent.parameter.id);
            deleteEntries(deleteIds);
        } catch (e) {
            deleteErrors = { errors: JSON.stringify([ e ]) };
            lWarn.logln(e);
        }
        new Event('event-push-notification.saved', deleteErrors).raise();
    }

} else if (raisedEvent.name === 'running') {
    lInfo.logln('Initializing App');
    Property.load();
    checkAndUpdateData();
    updateAllTriggers();
    fetchDeviceList();
}
