/* globals Ext, dss, _, createWaitWindow, getParameterByName */

/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
 *  Author: Sibylle Kohler
 */

var deviceArray = [];

function getStateGridTextActive (scene) {
    var sceneName = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextActive = Ext.String.format2(_('only when {name} is active'), { name: sceneName });
    return gridTextActive;
}

function getStateGridTextInactive (scene) {
    var sceneName = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextInactive = Ext.String.format2(_('only when {name} is inactive'), { name: sceneName });
    return gridTextInactive;
}

Ext.define('DSS.component.myTextedit', {
    extend: 'Ext.form.field.Text',
    allowBlank: false,
    valueToRaw: function (value) {
        return value == null
            ? ''
            : unescape(value);
    },
    rawToValue: function (value) {
        return value
            ? escape(value)
            : null;
    },
    constructor: function (config) {
        config.fieldLabel = _('Message');
        config.labelWidth = 100;
        config.padding = '0 0 0 10';
        config.style = {
            color: '#005138',
            'font-family': 'tahoma,arial,verdana,sans-serif'
        };
        config.width = 700;
        this.callParent(arguments);
    }
});

var ALL_RECEIVERS = '*';
var targets = {
    ALL: 'ALL_TARGETS',
    SPECIFIC: 'SPECIFIC_TARGETS'
};

Ext.define('DSS.component.MobileSelector', {
    extend: 'DSS.component.Base',
    checkboxArray: [],
    mixins: {
        observable: 'Ext.util.Observable'
    },
    constants: {
        OWN_HEIGHT: 150,
        PANEL_HEIGHT: 240,
        MAX_DEVICES: 8
    },
    getHeight: function () {
        return this.constants.OWN_HEIGHT;
    },
    updateCheckbox: function () {
        var me = this;
        this.checkboxArray = [];
        this.basePanel.removeAll();
        if (deviceArray == null || deviceArray.length == null) {
            return;
        }
        this.checkboxArray = deviceArray.map(function (device) {
            var checkbox = Ext.create('Ext.form.field.Checkbox', {
                id: device.uuid,
                boxLabel: device.name + ': ' + device.appName,
                style: {
                    color: '#005138',
                    'font-family': 'tahoma,arial,verdana,sans-serif'
                }
            });
            return checkbox;
        });
        this.checkboxArray.forEach(function (checkbox) {
            me.basePanel.add(checkbox);
        });
        if ((deviceArray.length) > this.constants.MAX_DEVICES) {
            this.basePanel.setHeight(this.constants.PANEL_HEIGHT);
        }
    },
    basePanel: null,
    targetDevicesRadioGroup: null,
    constructor: function () {
        var me = this;
        this.basePanel = Ext.create('Ext.form.FieldSet', {
            padding: '5 0 0 10',
            margin: '0 70 0 126',
            title: _('choose devices'),
            autoScroll: true
        });
        this.targetDevicesRadioGroup = Ext.create('Ext.form.RadioGroup', {
            fieldLabel: _('target devices:'),
            columns: 1,
            vertical: true,
            style: {
                color: '#005138',
                'font-family': 'tahoma,arial,verdana,sans-serif'
            },
            items: [
                { boxLabel: _('all devices'), name: 'targetDeviceRadioButtons', inputValue: targets.ALL },
                { boxLabel: _('specific devices'), name: 'targetDeviceRadioButtons', inputValue: targets.SPECIFIC }
            ],
            listeners: {
                change: function () {
                    if (this.getValue().targetDeviceRadioButtons === targets.SPECIFIC) {
                        me.basePanel.setDisabled(false);
                    } else {
                        me.basePanel.setDisabled(true);
                    }
                }
            }
        });
        this.checkboxArray = [];
        Ext.apply(this, {
            padding: '0 0 0 10',
            layout: 'auto',
            items: [
                this.targetDevicesRadioGroup,
                this.basePanel
            ]
        });
        this.targetDevicesRadioGroup.setValue({ targetDeviceRadioButtons: targets.ALL });
        this.callParent(arguments);
    },
    reset: function () {
        this.checkboxArray.forEach(function (checkbox) {
            checkbox.setValue(false);
        });
    },
    setValue: function (targetsToSet) {
        this.reset();
        var allTargets = targetsToSet[0] && targetsToSet[0] === ALL_RECEIVERS;
        if (!allTargets) {
            this.checkboxArray.forEach(function (checkbox) {
                if (targetsToSet.indexOf(checkbox.getId()) !== -1) {
                    checkbox.setValue(true);
                }
            });
        }
        this.targetDevicesRadioGroup.setValue({ targetDeviceRadioButtons: allTargets ? targets.ALL : targets.SPECIFIC });
    },
    isValid: function () {
        if (this.targetDevicesRadioGroup.getValue().targetDeviceRadioButtons === targets.ALL) {
            return true;
        }
        for (var i = 0; i < this.checkboxArray.length; i++) {
            if (this.checkboxArray[i].getValue()) {
                return true;
            }
        }
        return false;
    },
    getValue: function () {
        if (this.targetDevicesRadioGroup.getValue().targetDeviceRadioButtons === targets.SPECIFIC) {
            return this.checkboxArray
                .filter(function (checkbox) {
                    return checkbox.getValue();
                })
                .map(function (checkbox) {
                    return checkbox.getId();
                });
        }
        return [ ALL_RECEIVERS ];
    }
});


Ext.define('DSS.component.targetColumn', {
    extend: 'Ext.grid.column.Column',
    alias: 'widget.targetcolumn',
    renderer: function (values) {      // , metadata, record, rowIdx, colIdx, store, view
        var allTargets = Boolean(values && values[0] && values[0] === ALL_RECEIVERS);
        if (allTargets) {
            return _('all registered devices');
        }
        var tableRows = values
            .map(function (target) {
                var targetDevice = deviceArray && deviceArray.find(function (device) {
                    return device.uuid === target;
                });
                return targetDevice
                    ? '<tr><td>' + targetDevice.name + ': ' + targetDevice.appName + '</td></tr>'
                    : '<tr><td style="color:#ff0000;">' + _('Device not connected') + '</td></tr>';
            })
            .join('');
        return '<table>' + tableRows + '</table>';
    }
});

var ACTIVE = '1';
var INACTIVE = '2';
var ALARM1_SCENE = 74;
var ALARM2_SCENE = 83;
var ALARM3_SCENE = 84;
var ALARM4_SCENE = 85;

Ext.define('DSS.addon.EventPushNotification', {

    extend: 'DSS.addon.Framework',

    config: {
        appName: _('Event-Push-Notification'),
        appId: 'event-push-notification',
        appVersion: '1.0.7',
        dSSVersion: '1.9.0',
        appIcon: 'default_icon.png'
    },

    getHelp: function () {
        return Ext.create('Ext.Component', {
            whiteSpace: 'normal',
            padding: '0 20 0 0', // top right bottom left
            loader: {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad: true,
                disableCaching: false
            }
        });
    },

    getContent: function () {
        var me = this;
        this.mobileSelector = Ext.create('DSS.component.MobileSelector', {});

        var conditions = [
            {
                type: 'singleItem',
                title: _('state_daynight'),
                entry: [
                    {
                        type: 'state',
                        name: 'daylight',
                        value: true,
                        title: Ext.String.format2(_('daynight_bright'), {
                            duskTime: dss.staticDataModel.getDusk(),
                            dawnTime: dss.staticDataModel.getDawn()
                        }),
                        gridText: Ext.String.format2(_('grid_daynight_twilight_true'), {
                            duskTime: dss.staticDataModel.getDusk(),
                            dawnTime: dss.staticDataModel.getDawn()
                        })
                    }, {
                        type: 'state',
                        name: 'daylight',
                        value: false,
                        title: Ext.String.format2(_('daynight_dark'), {
                            duskTime: dss.staticDataModel.getDusk(),
                            dawnTime: dss.staticDataModel.getDawn()
                        }),
                        gridText: Ext.String.format2(_('grid_daynight_twilight_false'), {
                            duskTime: dss.staticDataModel.getDusk(),
                            dawnTime: dss.staticDataModel.getDawn()
                        })
                    }, {
                        type: 'state',
                        name: 'daynight',
                        value: true,
                        title: Ext.String.format2(_('daynight_day'), {
                            sunriseTime: dss.staticDataModel.getSunrise(),
                            sunsetTime: dss.staticDataModel.getSunset()
                        }),
                        gridText: Ext.String.format2(_('grid_daynight_daynight_true'), {
                            sunriseTime: dss.staticDataModel.getSunrise(),
                            sunsetTime: dss.staticDataModel.getSunset()
                        })
                    }, {
                        type: 'state',
                        name: 'daynight',
                        value: false,
                        title: Ext.String.format2(_('daynight_night'), {
                            sunriseTime: dss.staticDataModel.getSunrise(),
                            sunsetTime: dss.staticDataModel.getSunset()
                        }),
                        gridText: Ext.String.format2(_('grid_daynight_daynight_false'), {
                            sunriseTime: dss.staticDataModel.getSunrise(),
                            sunsetTime: dss.staticDataModel.getSunset()
                        })
                    }
                ]
            },
            { type: 'timeframe', title: _('Timeframe') },
            { type: 'weekdays', title: _('Weekdays') },
            { type: 'dateframe', title: _('Dateframe') },
            { type: 'deviceState', title: _('automatic devices') },
            { type: 'vdcDeviceState', title: _('device states') },
            { type: 'userDefinedState', title: _('Custom states') },
            { type: 'globalApplication', title: _('Global Application states') },
            {
                type: 'singleItem',
                title: _('state_presence'),
                entry: [
                    { type: 'state', name: 'presence', value: 'present', title: _('presence_present'), gridText: _('grid_presence_present') },
                    { type: 'state', name: 'presence', value: 'absent', title: _('presence_absent'), gridText: _('grid_presence_absent') }
                ]
            }, {
                type: 'singleItem',
                title: _('state_holiday'),
                entry: [
                    { type: 'state', name: 'holiday', value: 'on', title: _('holiday_on'), gridText: _('grid_holiday_on') },
                    { type: 'state', name: 'holiday', value: 'off', title: _('holiday_off'), gridText: _('grid_holiday_off') }
                ]
            }, {
                type: 'multiItem',
                title: _('weather trigger'),
                entry: [
                    { type: 'state', name: 'rain', value: ACTIVE, title: _('rain active'), gridText: _('grid_rain_active'), group: 1 },
                    { type: 'state', name: 'rain', value: INACTIVE, title: _('rain inactive'), gridText: _('grid_rain_inactive'), group: 1 },
                    { type: 'state', name: 'wind', value: ACTIVE, title: _('wind active'), gridText: _('grid_wind_active'), group: 2 },
                    { type: 'state', name: 'wind', value: INACTIVE, title: _('wind inactive'), gridText: _('grid_wind_inactive'), group: 2 },
                    { type: 'state', name: 'hail', value: ACTIVE, title: _('hail active'), gridText: _('grid_hail_active'), group: 3 },
                    { type: 'state', name: 'hail', value: INACTIVE, title: _('hail inactive'), gridText: _('grid_hail_inactive'), group: 3 },
                    { type: 'state', name: 'frost', value: ACTIVE, title: _('frost active'), gridText: _('grid_frost_active'), group: 4 },
                    { type: 'state', name: 'frost', value: INACTIVE, title: _('frost inactive'), gridText: _('grid_frost_inactive'), group: 4 },
                    { type: 'state', name: 'sun', value: ACTIVE, title: _('sun active'), gridText: _('grid_sun_active'), group: 5 },
                    { type: 'state', name: 'sun', value: INACTIVE, title: _('sun inactive'), gridText: _('grid_sun_inactive'), group: 5 }
                ]
            },
            { type: 'state', name: 'service', value: ACTIVE, title: _('service active'), gridText: _('grid_service_active'), group: 6 },
            { type: 'state', name: 'service', value: INACTIVE, title: _('service inactive'), gridText: _('grid_service_inactive'), group: 6 },
            {
                type: 'multiItem',
                title: _('Security'),
                entry: [
                    { type: 'state', name: 'fire', value: ACTIVE, title: _('fire active'), gridText: _('grid_fire_active'), group: 1 },
                    { type: 'state', name: 'fire', value: INACTIVE, title: _('fire inactive'), gridText: _('grid_fire_inactive'), group: 1 },
                    { type: 'state', name: 'panic', value: ACTIVE, title: _('panic_active'), gridText: _('grid_panic_active'), group: 2 },
                    { type: 'state', name: 'panic', value: INACTIVE, title: _('panic_inactive'), gridText: _('grid_panic_inactive'), group: 2 },
                    { type: 'state', name: 'alarm', value: ACTIVE, title: getStateGridTextActive(ALARM1_SCENE), gridText: getStateGridTextActive(ALARM1_SCENE), group: 3 },
                    { type: 'state', name: 'alarm', value: INACTIVE, title: getStateGridTextInactive(ALARM1_SCENE), gridText: getStateGridTextInactive(ALARM1_SCENE), group: 3 },
                    { type: 'state', name: 'alarm2', value: ACTIVE, title: getStateGridTextActive(ALARM2_SCENE), gridText: getStateGridTextActive(ALARM2_SCENE), group: 4 },
                    { type: 'state', name: 'alarm2', value: INACTIVE, title: getStateGridTextInactive(ALARM2_SCENE), gridText: getStateGridTextInactive(ALARM2_SCENE), group: 4 },
                    { type: 'state', name: 'alarm3', value: ACTIVE, title: getStateGridTextActive(ALARM3_SCENE), gridText: getStateGridTextActive(ALARM3_SCENE), group: 5 },
                    { type: 'state', name: 'alarm3', value: INACTIVE, title: getStateGridTextInactive(ALARM3_SCENE), gridText: getStateGridTextInactive(ALARM3_SCENE), group: 5 },
                    { type: 'state', name: 'alarm4', value: ACTIVE, title: getStateGridTextActive(ALARM4_SCENE), gridText: getStateGridTextActive(ALARM4_SCENE), group: 6 },
                    { type: 'state', name: 'alarm4', value: INACTIVE, title: getStateGridTextInactive(ALARM4_SCENE), gridText: getStateGridTextInactive(ALARM4_SCENE), group: 6 }
                ]
            },
            { type: 'zonestate', title: _('ZoneState') },
            { type: 'clusterstate', title: _('Cluster State') }
        ];

        var climateSettings = dss.staticDataModel.getClimateSettings();
        if (climateSettings && climateSettings.capabilities && climateSettings.capabilities.coolingSupported) {
            conditions.push({
                type: 'climateState',
                title: _('Climate system'),
                entry: [
                    { type: 'addon-state', name: 'apartment_controller', value: '0', title: _('off'), gridText: _('grid_climate_off'), group: 1 },
                    { type: 'addon-state', name: 'apartment_controller', value: '1', title: _('heating on'), gridText: _('grid_climate_heating'), group: 1 },
                    { type: 'addon-state', name: 'apartment_controller', value: '2', title: _('cooling on'), gridText: _('grid_climate_cooling'), group: 1 }
                ]
            });
        } else {
            conditions.push({
                type: 'climateState',
                title: _('Climate system'),
                entry: [
                    { type: 'addon-state', name: 'apartment_controller', value: '0', title: _('off'), gridText: _('grid_climate_off'), group: 1 },
                    { type: 'addon-state', name: 'apartment_controller', value: '1', title: _('heating on'), gridText: _('grid_climate_heating'), group: 1 }
                ]
            });
        }

        function customRenderer (val, metadata, record, rowIdx, colIdx, store, view) {
            var customColumnRenderer = view.up().customColumnRenderer;
            if (customColumnRenderer && customColumnRenderer[colIdx]) {
                var sReturn = customColumnRenderer[colIdx](val, metadata, record, rowIdx, colIdx, store, view);
                if (sReturn != null) {
                    return sReturn;
                }
            }
            var fieldActive = view.up().isFieldActive(record);
            return fieldActive
                ? Ext.String.htmlEncode(unescape(val))
                : '<span style="color:grey;">' + Ext.String.htmlEncode(unescape(val)) + '</span>';
        }

        function getEntryIds (entryOrEntries) {
            var isSingleEntry = entryOrEntries.get != null;
            return isSingleEntry
                ? entryOrEntries.get('id')
                : entryOrEntries
                    .filter(function (entry) {
                        return entry != null;
                    })
                    .map(function (entry) {
                        return entry.get('id');
                    });
        }

        var config = {
            defaultSortCol: 'name',
            disabledProperty: 'disabled',
            storeConfig: {
                storeId: 'pushStore',
                fields: [
                    { name: 'disabled', sortType: 'asBoolean' },
                    { name: 'name', sortType: 'asUCText' },
                    { name: 'uuid', sortType: 'asUCText' },
                    { name: 'id' },
                    { name: 'message', sortType: 'asUCText' },
                    { name: 'triggers', sortType: 'asTrigger' },
                    { name: 'conditions', sortType: 'asUCText' },
                    { name: 'grouping', sortType: 'asUCText' },
                    { name: 'defaultFooter', sortType: 'asBoolean' }
                ]
            },
            multiselect: true,
            columns: [
                {
                    header: _('Active'),
                    dataIndex: 'disabled',
                    resizable: false,
                    xtype: 'disabledcolumn',
                    width: 50
                }, {
                    header: _('Name'),
                    dataIndex: 'name',
                    xtype: 'textcolumn',
                    width: 200,
                    renderer: customRenderer
                }, {
                    header: _('Trigger'),
                    dataIndex: 'triggers',
                    maintainFlex: true,
                    flex: 1,
                    xtype: 'triggercolumn'
                }, {
                    header: _('Target'),
                    dataIndex: 'uuid',
                    maintainFlex: true,
                    flex: 1,
                    xtype: 'targetcolumn'
                }, {
                    header: _('Message'),
                    dataIndex: 'message',
                    xtype: 'textcolumn',
                    flex: 1,
                    renderer: customRenderer
                }, {
                    header: _('Conditions'),
                    dataIndex: 'conditions', // alibi index
                    xtype: 'conditioncolumn',
                    maintainFlex: true,
                    flex: 1,
                    customRendererOptions: {
                        hideEnabled: true,
                        hideWeekdays: false,
                        hideGenericSystemStates: false,
                        hideZoneStates: false,
                        hideTimeframe: false,
                        filterSystemState: [],
                        stateTitles: [
                            { name: 'daylight', value: true, title: _('grid_daylight_true') },
                            { name: 'daylight', value: false, title: _('grid_daylight_false') },
                            { name: 'daynight', value: true, title: _('grid_daynight_true') },
                            { name: 'daynight', value: false, title: _('grid_daynight_false') },
                            { name: 'panic', value: ACTIVE, title: _('grid_panic_active') },
                            { name: 'panic', value: INACTIVE, title: _('grid_panic_inactive') },
                            { name: 'presence', value: 'present', title: _('grid_presence_present') },
                            { name: 'presence', value: 'absent', title: _('grid_presence_absent') },
                            { name: 'fire', value: ACTIVE, title: _('grid_fire_active') },
                            { name: 'fire', value: INACTIVE, title: _('grid_fire_inactive') },
                            { name: 'hail', value: ACTIVE, title: _('grid_hail_active') },
                            { name: 'hail', value: INACTIVE, title: _('grid_hail_inactive') },
                            { name: 'wind', value: ACTIVE, title: _('grid_wind_active') },
                            { name: 'wind', value: INACTIVE, title: _('grid_wind_inactive') },
                            { name: 'rain', value: ACTIVE, title: _('grid_rain_active') },
                            { name: 'rain', value: INACTIVE, title: _('grid_rain_inactive') },
                            { name: 'frost', value: ACTIVE, title: _('grid_frost_active') },
                            { name: 'frost', value: INACTIVE, title: _('grid_frost_inactive') },
                            { name: 'alarm', value: ACTIVE, title: getStateGridTextActive(ALARM1_SCENE) },
                            { name: 'alarm', value: INACTIVE, title: getStateGridTextInactive(ALARM1_SCENE) },
                            { name: 'alarm2', value: ACTIVE, title: getStateGridTextActive(ALARM2_SCENE) },
                            { name: 'alarm2', value: INACTIVE, title: getStateGridTextInactive(ALARM2_SCENE) },
                            { name: 'alarm3', value: ACTIVE, title: getStateGridTextActive(ALARM3_SCENE) },
                            { name: 'alarm3', value: INACTIVE, title: getStateGridTextInactive(ALARM3_SCENE) },
                            { name: 'alarm4', value: ACTIVE, title: getStateGridTextActive(ALARM4_SCENE) },
                            { name: 'alarm4', value: INACTIVE, title: getStateGridTextInactive(ALARM4_SCENE) },
                            { name: 'holiday', value: 'on', title: _('grid_holiday_on') },
                            { name: 'holiday', value: 'off', title: _('grid_holiday_off') },
                            { name: 'heating_water_system', value: '0', title: _('grid_climate_off') },
                            { name: 'heating_water_system', value: '1', title: _('grid_climate_heating') },
                            { name: 'heating_water_system', value: '2', title: _('grid_climate_cooling') }
                        ]
                    }
                }
            ],
            buttonText: {
                newButton: _('New message'),
                editButton: _('Edit message'),
                enableButton: _('Activate message'),
                disableButton: _('Deactivate message'),
                testButton: _('Test message'),
                deleteButton: _('Delete message')
            },
            editorConfig: {
                title: _('Messageproperties'),
                staticControls: [
                    {
                        type: 'text',
                        label: _('Name'),
                        id: 'name'
                    }, {
                        type: 'hidden',
                        label: _('id'),
                        id: 'id'
                    }
                ],
                tabControls: [
                    {
                        tabLabel: _('Trigger'),
                        tabDescription: _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
                        content: [
                            {
                                type: 'TriggerConfigurator',
                                id: 'triggers',
                                flex: true
                            }
                        ]
                    }, {
                        tabLabel: _('Message'),
                        tabDescription: _('Provide the Message, which will be send.'),
                        content: [
                            {
                                type: 'custom',
                                id: 'message',
                                label: _('Message'),
                                control: Ext.create('DSS.component.myTextedit', {})
                            }, {
                                type: 'custom',
                                label: _('Target'),
                                id: 'uuid',
                                control: this.mobileSelector,
                                flex: true
                            }
                        ]
                    }, {
                        tabLabel: _('Conditions'),
                        tabDescription: _('Choose the conditions.'),
                        content: [
                            {
                                type: 'ConditionConfigurator',
                                id: 'conditions',
                                conditions: conditions,
                                flex: true
                            }
                        ]
                    }
                ]
            },
            listeners: {
                loadDataStore: function (grid) {
                    me.refetchDevices(grid);
                },
                saveEntry: function (grid, values) {
                    me.waitAndReload(grid, {
                        actions: 'save',
                        value: Ext.JSON.encode(values)
                    });
                },
                enableEntry: function (grid, entries) {
                    me.waitAndReload(grid, {
                        actions: 'enable',
                        id: Ext.JSON.encode(getEntryIds(entries))
                    });
                },
                disableEntry: function (grid, entries) {
                    me.waitAndReload(grid, {
                        actions: 'disable',
                        id: Ext.JSON.encode(getEntryIds(entries))
                    });
                },
                deleteEntry: function (grid, entries) {
                    me.waitAndReload(grid, {
                        actions: 'delete',
                        id: Ext.JSON.encode(getEntryIds(entries))
                    });
                },
                testEntry: function (grid, entry) {
                    dss.raiseEvent('event-push-notification', {
                        actions: 'test',
                        id: entry.get('id')
                    });
                }
            }
        };

        if (getParameterByName('mode') === 'expert') {
            config.grouperField = 'grouping';
            config.editorConfig.tabControls[0].content[0].userStates = true;
        }

        return Ext.create('DSS.gridPanel', config);
    },
    loadDevices: function () {
        var response = dss.ajaxSyncRequest('/json/property/query', {
            query: '/scripts/event-push-notification/devices/*(name,uuid,appName)'
        });
        var data = Ext.JSON.decode(response);
        if (data.result.devices == null) {
            Ext.MessageBox.alert(
                _('No target devices are known'),
                _('No target devices are known. Please turn on the my.digitalstrom app and register some smartphones.')
            );
            return;
        }
        deviceArray = data.result.devices;
        this.mobileSelector.updateCheckbox();
    },
    refetchDevices: function (grid) {
        var waitWindow = createWaitWindow();
        grid.setStatus(_('Fetching devicelist from cloud ...'));
        waitWindow.showWindow(_('Fetching devicelist from cloud ...'), true);
        var me = this;

        dss.pushAndQuery('event-push-notification', { actions: 'fetchDeviceList' }, 'event-push-notification.saved', function (eventData) {
            grid.setStatus(_('Refreshing device list'));
            me.loadDevices();
            me.loadParameter(grid);
            waitWindow.closeWindow();
            grid.setStatus(_('Ready'));
            if (eventData.properties && eventData.properties.errors && eventData.properties.errors.length) {
                var errorMessagesTranslated = JSON.parse(eventData.properties.errors);
                errorMessagesTranslated = errorMessagesTranslated
                    .filter(function (error, index) {       // remove possible duplicates
                        return errorMessagesTranslated.indexOf(error) === index;
                    })
                    .map(function (error) {
                        return _(error);
                    });
                Ext.MessageBox.alert(_('Error occurred'), errorMessagesTranslated.join(' ') + '<br/>' + _('Please try again later.'));
            }
            return true;
        });
    },
    waitAndReload: function (grid, parameter) {
        grid.setStatus(_('Saving ...'));
        var me = this;

        dss.pushAndQuery('event-push-notification', parameter, 'event-push-notification.saved', function () {       // eventData
            me.loadParameter(grid);
            return true;
        });
    },
    loadParameter: function (grid) {
        grid.setStatus(_('Loading ...'));
        var response = dss.ajaxSyncRequest('/json/property/query', {
            query: '/scripts/event-push-notification/values/*(*)/*(*)/*(*)/*(*)/*(*)'
        });
        var data = Ext.JSON.decode(response);

        var rawPropTreeEntries = data.result.values;
        var parsedMessageEntries = dss.parsePropertyQueryAnswer(data, [
            { name: 'name', type: 'simple' },
            { name: 'id', type: 'simple' },
            { name: 'disabled', type: 'condition/enabled' },
            { name: 'message', type: 'simple' },
            { name: 'triggers', type: 'trigger' },
            { name: 'conditions', type: 'condition' }
        ]);
        if (rawPropTreeEntries && parsedMessageEntries) {

            rawPropTreeEntries.forEach(function (entry) {
                var id = Number(entry.id);
                var uuids = [];
                var uuidsContainer = entry[id].find(function (item) {
                    return item.uuid != null;
                });
                if (uuidsContainer) {
                    for (var key in uuidsContainer) {
                        if (key === 'uuid') {
                            continue;
                        }
                        uuids.push(uuidsContainer[key]);
                    }
                }
                parsedMessageEntries = parsedMessageEntries.map(function (item) {
                    if (Number(item.id) === id) {
                        item.uuid = uuids;
                        item.grouping = unescape(item.name).split(' ')[0];
                    }
                    return item;
                });
            });

            grid.loadParameter(parsedMessageEntries);
        }
        grid.setStatus(_('Ready'));
    }
});

Ext.onReady(function () {
    dss.buildUpLang(['locale/{languageSuffix}/event-push-notification.po']);
    var eventPushNotification = Ext.create('DSS.addon.EventPushNotification');
    eventPushNotification.initPage();
});
